{
    "$graph": [
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap-merge:v0.0.3"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string",
                        {
                            "type": "array",
                            "items": "string"
                        }
                    ],
                    "inputBinding": {
                        "prefix": "--aliases"
                    },
                    "doc": "Unique aliases for feature files to be used as names for the --report columns\nin the merged file.\nDefault: basenames of files provided in --features without extensions\n",
                    "id": "#feature-merge.cwl/feature_aliases"
                },
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "prefix": "--features"
                    },
                    "doc": "CSV/TSV feature files to be merged\n",
                    "id": "#feature-merge.cwl/feature_files"
                },
                {
                    "type": [
                        "null",
                        "string",
                        {
                            "type": "array",
                            "items": "string"
                        }
                    ],
                    "inputBinding": {
                        "prefix": "--mergeby"
                    },
                    "doc": "Column names to merge feature files by.\nDefault: RefseqId, GeneId, Chrom, TxStart, TxEnd, Strand\n",
                    "id": "#feature-merge.cwl/mergeby"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--output"
                    },
                    "doc": "Output file prefix.\nDefault: merged_\n",
                    "id": "#feature-merge.cwl/output_prefix"
                }
            ],
            "baseCommand": [
                "run_merge.R"
            ],
            "stdout": "merge_stdout.log",
            "stderr": "merge_stderr.log",
            "label": "Feature merge - merges feature files based on the specified columns",
            "doc": "Tool merges input feature files based on the columns provided in --mergeby\ninput. All input feature CSV/TSV files should have the header (case-sensitive)\nFormat of the input files is identified based on file's extension\n*.csv - CSV\n*.tsv - TSV\nOtherwise used CSV by default\n\nThe output file's rows order corresponds to the rows order of the first\nCSV/TSV feature file. Output is always saved in TSV format.\n\nOutput file includes only rows intersected by column names set in --mergeby.\nOutput file includes only columns set in --mergeby and --report parameters.\nColumn set in the --report parameter is renamed based on the --aliases or \nbasenames of the --features files.\n",
            "id": "#feature-merge.cwl",
            "http://schema.org/name": "Feature merge - merges feature files based on the specified columns",
            "http://schema.org/alternateName": "Feature merge - merges feature files based on the specified columns",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/feature-merge.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: run_merge.R\n      [-h] -f FEATURES [FEATURES ...] [-a [ALIASES [ALIASES ...]]]\n      [-m [MERGEBY [MERGEBY ...]]] [-o OUTPUT]\n\nMerge feature files based on the specified columns\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -f FEATURES [FEATURES ...], --features FEATURES [FEATURES ...]\n                        CSV/TSV feature files to be merged\n  -a [ALIASES [ALIASES ...]], --aliases [ALIASES [ALIASES ...]]\n                        Unique aliases for feature files to be used as\n                        prefixes for not --mergeby columns in the merged file.\n                        Default: basenames of files provided in --features\n                        without extensions\n  -m [MERGEBY [MERGEBY ...]], --mergeby [MERGEBY [MERGEBY ...]]\n                        Column names to merge feature files by. Default:\n                        RefseqId, GeneId, Chrom, TxStart, TxEnd, Strand\n  -o OUTPUT, --output OUTPUT\n                        Output file prefix. Default: merged_\n",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*report.tsv"
                    },
                    "id": "#feature-merge.cwl/merged_file"
                },
                {
                    "type": "File",
                    "id": "#feature-merge.cwl/stderr_log",
                    "outputBinding": {
                        "glob": "merge_stderr.log"
                    }
                },
                {
                    "type": "File",
                    "id": "#feature-merge.cwl/stdout_log",
                    "outputBinding": {
                        "glob": "merge_stdout.log"
                    }
                }
            ]
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "sd:upstream": {
                "rnaseq_sample": [
                    "rnaseq-se.cwl",
                    "rnaseq-pe.cwl",
                    "rnaseq-se-dutp.cwl",
                    "rnaseq-pe-dutp.cwl",
                    "rnaseq-se-dutp-mitochondrial.cwl",
                    "rnaseq-pe-dutp-mitochondrial.cwl",
                    "trim-rnaseq-pe.cwl",
                    "trim-rnaseq-se.cwl",
                    "trim-rnaseq-pe-dutp.cwl",
                    "trim-rnaseq-pe-smarter-dutp.cwl",
                    "trim-rnaseq-se-dutp.cwl",
                    "trim-quantseq-mrnaseq-se.cwl"
                ]
            },
            "inputs": [
                {
                    "type": "string",
                    "label": "Experiment short name/Alias",
                    "sd:preview": {
                        "position": 1
                    },
                    "id": "#main/alias"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#main/group_by/isoforms",
                                "#main/group_by/genes",
                                "#main/group_by/common tss"
                            ]
                        }
                    ],
                    "default": "genes",
                    "label": "Group by",
                    "doc": "Grouping method for features: isoforms, genes or common tss",
                    "id": "#main/group_by"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "string"
                        }
                    ],
                    "default": [
                        "RefseqId",
                        "GeneId",
                        "Chrom",
                        "TxStart",
                        "TxEnd",
                        "Strand"
                    ],
                    "label": "Columns to merge experiments by",
                    "doc": "Column names to merge feature files by",
                    "sd:layout": {
                        "advanced": true
                    },
                    "id": "#main/merge_by_columns"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "default": null,
                    "format": "http://edamontology.org/format_3475",
                    "label": "RNA-Seq experiments",
                    "doc": "CSV/TSV file with RPKM grouped by common TSS",
                    "sd:upstreamSource": "rnaseq_sample/rpkm_common_tss",
                    "id": "#main/rpkm_common_tss"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "default": null,
                    "format": "http://edamontology.org/format_3475",
                    "label": "RNA-Seq experiments",
                    "doc": "CSV/TSV file with RPKM grouped by gene",
                    "sd:upstreamSource": "rnaseq_sample/rpkm_genes",
                    "id": "#main/rpkm_genes"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "default": null,
                    "format": "http://edamontology.org/format_3752",
                    "label": "RNA-Seq experiments",
                    "doc": "CSV/TSV file with RPKM grouped by isoform",
                    "sd:upstreamSource": "rnaseq_sample/rpkm_isoforms",
                    "sd:localLabel": true,
                    "id": "#main/rpkm_isoforms"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "label": "RNA-Seq experiments",
                    "doc": "Aliases for RNA-Seq experiments to be used as the names for the reported\ncolumns in the merged file. Order corresponds to the rpkm_genes input\n",
                    "sd:upstreamSource": "rnaseq_sample/alias",
                    "id": "#main/sample_names"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "Merging stderr log",
                    "doc": "Merging stderr log",
                    "outputSource": "#main/feature_merge/stderr_log",
                    "id": "#main/feature_merge_stderr_log"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "Merging stdout log",
                    "doc": "Merging stdout log",
                    "outputSource": "#main/feature_merge/stdout_log",
                    "id": "#main/feature_merge_stdout_log"
                },
                {
                    "type": "File",
                    "label": "Merged feature expression file",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Merged by merge_by_columns feature expression file\n",
                    "outputSource": "#main/feature_merge/merged_file",
                    "sd:visualPlugins": [
                        {
                            "syncfusiongrid": {
                                "tab": "Merged Feature Expression",
                                "Title": "Merged Feature Expression"
                            }
                        }
                    ],
                    "id": "#main/merged_file"
                }
            ],
            "steps": [
                {
                    "run": "#feature-merge.cwl",
                    "in": [
                        {
                            "source": "#main/sample_names",
                            "id": "#main/feature_merge/feature_aliases"
                        },
                        {
                            "source": [
                                "#main/group_by",
                                "#main/rpkm_isoforms",
                                "#main/rpkm_genes",
                                "#main/rpkm_common_tss"
                            ],
                            "valueFrom": "${\n    if (self[0] == \"isoforms\") {\n      return self[1];\n    } else if (self[0] == \"genes\") {\n      return self[2];\n    } else {\n      return self[3];\n    }\n}\n",
                            "id": "#main/feature_merge/feature_files"
                        },
                        {
                            "source": "#main/merge_by_columns",
                            "id": "#main/feature_merge/mergeby"
                        }
                    ],
                    "out": [
                        "#main/feature_merge/merged_file",
                        "#main/feature_merge/stdout_log",
                        "#main/feature_merge/stderr_log"
                    ],
                    "id": "#main/feature_merge"
                }
            ],
            "label": "Feature expression merge - combines feature expression from several experiments",
            "doc": "Feature expression merge - combines feature expression from several experiments\n=========================================================================\n\nWorkflows merges RPKM (by default) gene expression from several\nexperiments based on the values from GeneId, Chrom, TxStart, TxEnd and\nStrand columns (by default). Reported unique columns are renamed based on\nthe experiments names.\n",
            "id": "#main",
            "http://schema.org/name": "Feature expression merge - combines feature expression from several experiments",
            "http://schema.org/alternateName": "Feature expression merge - combines gene expression from several experiments",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/datirium/workflows/master/workflows/feature-merge.cwl",
            "http://schema.org/codeRepository": "https://github.com/datirium/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    ],
    "cwlVersion": "v1.0",
    "$schemas": [
        "https://github.com/schemaorg/schemaorg/raw/main/data/releases/11.01/schemaorg-current-http.rdf"
    ],
    "$namespaces": {
        "s": "http://schema.org/"
    }
}
